var QuickSum = {
  currentFunction: "sum",
  showing: false,
  values: [],

  onMouseUp: function(e) {
    var sel = getBrowser().contentWindow.getSelection();
    // only interested if there is a selection
    if(sel && sel.rangeCount > 0) {
      QuickSum.values = [];
      // grab all the range parts
      for(var i=0; i<sel.rangeCount; i++) {
        var r = sel.getRangeAt(i).toString();
        // strip off non-numeric stuff at both ends
        r = r.replace(/^[^-\d]+/g, '');
        r = r.replace(/[^\d]+$/g, '');
        if(r == '') // oops, nothing left!
          continue;

        // get rid of thousands separators
        // first try to figure out if we're in european format
        // this is totally a hack, we're just looking for > 1 period
        // or a period followed by a comma
        var p = r.indexOf('.');
        if(r.indexOf('.',p+1) > 0 || r.indexOf(',',p+1) > 0) {
          // probably euro
          r = r.replace(/\./g,'');
          r = r.replace(/,/g,'.');
        }
        // get rid of commas
        r = r.replace(/,/g, '');
        // get rid of spaces
        r = r.replace(/\s/g, '');
        r = parseFloat(r);
        
        if(isNaN(r)) // somehow bogus
          continue;

        QuickSum.values.push(r);
      }

      if(QuickSum.values.length > 0) {
        QuickSum.showing = true;
        // now apply the current function
        QuickSum.calcValue();
      }
      else {
        // nothing numeric selected
        QuickSum.showing = false;
        QuickSum.values = [];
        QuickSum.hideStatusPanel();
      }
    }
    else {
      // no selection, don't bother
      QuickSum.showing = false;
      QuickSum.values = [];
      QuickSum.hideStatusPanel();
    }
  },

  calcValue: function() {
    var v = QuickSum[QuickSum.currentFunction](QuickSum.values);
    // round to 4 decimal places
    v = Math.round(v * 10000) / 10000;
    // grab current function name from menuitem, stick it all in statustext
    var fn = document.getElementById('quicksum-' + QuickSum.currentFunction).label;
    QuickSum.setStatusText(fn + ': ' + v.toString());
  },

  setCommand: function(mi) {
    QuickSum.currentFunction = mi.id.replace(/^quicksum-/,'');
    if(QuickSum.showing) {
      QuickSum.calcValue();
    }
  },

  hideStatusPanel: function() {
    document.getElementById("quicksum-panel").collapsed = true;
  },

  setStatusText: function(t) {
    var s = document.getElementById("quicksum-panel");
    s.collapsed = false;
    s.label = t;
  },

  onUnload: function() {
    window.removeEventListener("onmouseup", QuickSum.onMouseUp, false);
  },

  // actual numeric functions
  sum: function(l) {
    var s = 0;
    l.forEach(function(x) { s += x; });
    return s;
  },

  average: function(l) {
    return QuickSum.sum(l) / l.length;
  },

  min: function(l) {
    return Math.min.apply(Math, l);
  },

  max: function(l) {
    return Math.max.apply(Math, l);
  }
};

window.addEventListener("mouseup", QuickSum.onMouseUp, false);
window.addEventListener("unload", QuickSum.onUnload, false);
